package com.tenlionsoft.aimz_k.viewmodel

import android.app.Application
import android.text.Editable
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.viewModelScope
import com.atwa.filepicker.result.ImageMeta
import com.tenlionsoft.aimz_k.model.UserAvatarBean
import com.tenlionsoft.aimz_k.model.UserInfoUpdateBean
import com.tenlionsoft.aimz_k.net.BaseApi
import com.tenlionsoft.baselib.base.BaseViewModel
import com.tenlionsoft.baselib.contacts.toBody
import com.tenlionsoft.baselib.contacts.toFileBody
import com.tenlionsoft.baselib.model.ModelEvent
import com.tenlionsoft.baselib.net.ExParse
import com.tenlionsoft.baselib.net.RetrofitClient
import com.tenlionsoft.baselib.utils.RegexUtils
import com.tenlionsoft.baselib.utils.SpUtils
import com.tenlionsoft.baselib.utils.ToastUtils
import kotlinx.coroutines.launch

class UserInfoEditViewModel(val context: Application) : BaseViewModel() {
    val phone = MutableLiveData<String>()
    val email = MutableLiveData<String>()
    val nickName = MutableLiveData<String>()
    val remark = MutableLiveData<String>()
    val isUpload = MutableLiveData<Boolean>()
    private val retrofitClient = RetrofitClient.getInstance(context)
    private val baseApi = retrofitClient.create(BaseApi::class.java)

    init {
        doGetMyInfo()
    }

    //获取我的信息
    private fun doGetMyInfo() {
        viewModelScope.launch {
            try {
                showLoad.value = ModelEvent(true, "获取中...")
                val userInfo = retrofitClient.makeApiCall {
                    baseApi.getUserInfo(SpUtils.getId())
                }
                if (userInfo != null) {
                    phone.value = userInfo.phone
                    email.value = userInfo.email
                    nickName.value = userInfo.nickname
                    remark.value = userInfo.remarks
                }
                showLoad.value = ModelEvent(false, "获取成功...")
            } catch (e: Exception) {
                e.printStackTrace()
                showLoad.value = ModelEvent(false, "获取失败...")
                ExParse.parse(e)
            }
        }
    }

    fun doUploadImg(meta: ImageMeta) {
        viewModelScope.launch {
            showLoad.value = ModelEvent(true, "上传中...")
            try {
                val responseBean = retrofitClient.makeApiCall {
                    val body = meta.file!!.toFileBody("image")
                    baseApi.doUploadImage(body)
                }
                if (responseBean.code == 200) {
                    //设置成功后刷新页面
                    val body =
                        UserAvatarBean(responseBean.data.fileId + ":" + responseBean.data.fileName).toBody()

                    val rBean = retrofitClient.makeApiCall {
                        baseApi.doUpdateUserAvatar(SpUtils.getId(), body)
                    }
                    if (rBean.code == 200) {
                        SpUtils.putAvatar(responseBean.data.fileId + ":" + responseBean.data.fileName)
                        isUpload.value = true
                        showLoad.value = ModelEvent(false, "上传成功")
                    } else {
                        isUpload.value = false
                        showLoad.value = ModelEvent(false, "上传成功")
                    }
                } else {
                    isUpload.value = false
                    showLoad.value = ModelEvent(false, "上传成功")
                }
            } catch (e: Exception) {
                e.printStackTrace()
                isUpload.value = false
                showLoad.value = ModelEvent(false, "失败")
                ExParse.parse(e)
            }

        }
    }


    //保存信息
    fun doSave() {
        val isLegal = checkParams()
        if (isLegal) {

            viewModelScope.launch {
                showLoad.value = ModelEvent(true, "保存中...")
                try {
                    val body = UserInfoUpdateBean(
                        avatar = SpUtils.getAvatar(),
                        email = email.value!!,
                        phone = phone.value!!,
                        password = "",
                        remarks = if (remark.value.isNullOrBlank()) "" else remark.value!!,
                        nickname = nickName.value!!
                    ).toBody()
                    val response = retrofitClient.makeApiCall {
                        baseApi.doUpdateUserInfo(SpUtils.getId(), body)
                    }
                    showLoad.value = ModelEvent(false, "保存中...")
                    if (response.code == 200) {
                        ToastUtils.success("更新成功")
                    } else {
                        ToastUtils.error(
                            response.msg
                                ?: "更新失败"
                        )
                    }
                } catch (e: Exception) {
                    e.printStackTrace()
                    showLoad.value = ModelEvent(false, "保存中...")
                    ExParse.parse(e)
                }
            }
        }
    }

    fun changeNickName(e: Editable) {
        this.nickName.value = e.toString()
    }

    fun changePhone(e: Editable) {
        this.phone.value = e.toString()
    }

    fun changeEmail(e: Editable) {
        this.email.value = e.toString()
    }

    fun changeRemark(e: Editable) {
        this.remark.value = e.toString()
    }

    private fun checkParams(): Boolean {
        if (phone.value.isNullOrEmpty() || !RegexUtils.isTel(phone.value)) {
            ToastUtils.error("联系电话有误")
            return false
        }
        if (email.value.isNullOrEmpty() || !RegexUtils.isEmail(email.value)) {
            ToastUtils.error("邮箱有误")
            return false
        }

        return true
    }
}